# 日志上报库分析文档

## 概述
该日志上报库为多平台统一接入层，支持 Android、iOS、小游戏/H5 等多端场景，整合了以下核心能力：
- **基础行为上报**：用户注册/登录/登出、支付等核心行为
- **广告转化跟踪**：激活、付费等广告效果指标
- **BI数据上报**：自定义业务指标采集
- **多SDK适配**：支持 TalkingData、TapDB、QuickSDK 等主流第三方平台
- 
## 配置：用户在部分小游戏渠道时需要在game_env.js的uploadConfig中配置开发者后台创建的id
### 示例
```ts
    var uploadConfig = {
        wx_minigame_id:{
            appid: "************************",
            secret_key: "******************************",
            action_set_id: "********************"
        }
    };
    winit.setProject(uploadConfig)
```
## 注意事项:
### 在使用微信小游戏的广告激活时，iOS小游戏端需要游戏主动上报支付事件（因为iOS小游戏支付前端只能帮忙拉起客服窗口，并不知道用户后续流程）
### 使用示例:  
```ts
 import {adgameUpload} from "pi_upload/bi/upload_ads/index"
```

## BI 核心类 `UploadContext`

### 类结构
```ts
export class UploadContext implements IUploadContext {
    BISdk: BISDK;               // BI 上报处理器           
    platforms: Map<string, IPlatformUpload>;   // 主流SDK实例（按平台分类）
    other_platforms: Map<string, IPlatformUpload>;  // 特殊平台实例（H5/小游戏）
    //...其他属性
}
```

#### 初始化配置
- **通过构造函数注入平台标识符数组，自动创建对应平台实例**：
```ts
import {UploadContext} from 'pi_upload/bi/index'

new UploadContext([
    UploadType.quicksdk, 
    UploadType.talkingdata,
    // 更多平台...
])
```
### 核心接口方法
####  用户生命周期上报
1. `register()`
**功能**: 处理用户注册事件
**上报内容**：
- 角色基础属性（等级、VIP等）
- 设备/渠道信息
- 触发广告激活事件（首次注册时

**调用场景**：
```ts
// 新用户完成注册后调用
uploadContext.register();

```

2. `login()`
**功能**: 处理用户登录事件
**上报差异**：
- 区别于注册时的 role.createTime 标记
- 登录频次统计（日活/月活）
- 触发广告激活事件（首次注册时

**调用场景**：
```ts
// 用户每次登录时调用
uploadContext.login()
```
3. `logout()`
**功能**: 处理用户登出事件
**特殊处理**：
- 清空会话级缓存数据
- 部分平台需要显式调用退出接口

**调用场景**：
```ts
// 用户登出时调用
uploadContext.logout()
```
4. `pay(payParam: UploadPayParam)`
**参数结构**: 
```ts
import { UploadPayParam } from "pi_upload/interface/pay";
interface UploadPayParam {
    orderId: string;     // 订单号
    amount: number;      // 支付金额（单位：分）
    currency: string;    // 货币类型（CNY/USD）
    productId: string;   // 商品ID
    paymentType: UploadPayType; // 支付方式（支付宝/微信等）
}
```
**上报逻辑**：
- 同步至所有配置的支付SDK
- 触发广告转化事件（LTV计算）

**调用场景**：
```ts
// 用户支付时调用
uploadContext.pay(param)
```
5. `custom(type: string, value?: Json)`
**功能**: 提交自定义业务指标

**调用示例**：
```ts
// 上报关卡完成事件
uploadContext.custom('level_complete', {
    level: 5,
    duration: 120
});
```

## GI 项目自定义方法上报，可用于各种平台

```ts
import {gameEventReporter} from "pi_upload/gi/index"
示例
gameEventReporter.levelUp({
    level: 5,
    duration: 120
});
```

### 全局环境变量设置
```ts
        let data = PISYS.Env.get('_$sso_user_info') as IUploadUser
        this.param = {
            serverID: data.serverID,
            serverName: data.serverName,
            userID : data.userID || "",      // 玩家（角色）ID，纯英文，唯一
            userName : data.userName || "",        // 玩家（角色）名；默认值：""
            userType : data.userType || "",        // 账号类型（来自哪个平台），比如：qq，wx，weibo 等；默认值：""
            userSex : data.userSex || "male",       // 玩家 性别；默认值："unknown"
            userAge : data.userAge || 0,      // 玩家 年龄；默认值：0
            roleLevel : data.roleLevel || 0,      // 角色等级；默认值：0
            roleVipLevel : data.roleVipLevel || 0,        // 用户 vip 充值 等级；默认值：0
            roleBalance : data.roleBalance || "",      // 角色 余额；默认值：""
            partyId : data.partyId || 0,      // 帮派id；默认值：0
            partyName : data.partyName || "",      // 帮派名；默认值：""
            createTime : data.createTime || 0,    // 创建时间；时间戳 默认值：0
            levelUpTime : data.levelUpTime || 0,      // 升级时间；时间戳 默认值：0
    
            device : await PISYS.Native.SystemInfo.getDeviceID(),      // 设备ID
            channel : await PISYS.Native.App.getFlavorName(),      // 渠道
            version : await PISYS.Native.App.getVersionCode() + "",     // APP 版本号
            network :  await PISYS.Native.Network.getType() + "",        // 网络环境
    
            ip : data.ip || "",        // 本机 IP
            uuid : data.uuid || "",         // 用户ID 唯一标识（不分区服）
            fightingCapacity : data.fightingCapacity || "",        // 战力
            uploadType : data.uploadType,
            payTotal : data.payTotal || 0,        //累充
            extend : data.extend || ""         // 扩展串；默认值：""
          };
```

### 结构说明
####     bi:包含游戏的所有默认上报功能
####      gi:包含游戏自定义上报功能
####     bi/upload_ads:包含游戏的激活广告上报，包括Android，iOS以及小游戏
####    bi/upload_sdks:包含游戏接入的SDK功能上报，非广告SDK
