/**
 * BI 系统前端统一封装
 * * 所有变量命名 - 蛇形命名(与后端接口对应)
 * * 对应后端文档 http://192.168.31.241:8181/docs/bi/bi-1bi688j2bg26e
 */

import { user_register_data, user_login_data, user_offline_data, user_pay_data, page_trace_data, device_use_data, user_device_data, gi_data_change_data, InterfaceType } from "./bi_base";
import { PISYS } from 'pi_sys/index';

interface IEnv {
    app_id: string;
    report?: boolean;
    os: { name: string, version: string },
    device: { type: string, platform: string, screen: { colorDepth: number, w: number, h: number } };
}

type page_trace_info = string | number;

type BI_DATA = user_register_data | user_login_data | user_offline_data | user_pay_data
    | user_device_data | device_use_data | gi_data_change_data | page_trace_data;

export namespace BIManager {
    let app_id: string;
    let device_id: string;
    let device_os: string;
    let device_model: string;
    let device_total_memory: number;
    let baseUrl: string = 'http://39.98.75.215:8099/kafka_port/';
    
    const page_trace_list: page_trace_info[] = [];
    let cur_page_trace_length = 0;
    let limit_report_length = 4000;
    let init_flag: boolean = false;

    let localSaveFlagCall: (key, value) => void;
    let readFlagCall: (key, value) => Promise<any>;

    /**
     * 修改 一次 report 数据大小限制
     * @param len 
     */
    export function changeReportLimitLength(len: number) {
        limit_report_length = len;
    }

    /**
     * 修改目标 BI 服务器地址
     * @example 'http://39.98.75.215:8099/kafka_port/'
     * @param url 
     */
    export function changeReportBaseUrl(url: string) {
        baseUrl = url;
    }

    export function Init(env: IEnv, saveFlagCall: (key, value) => void, readFlagCall: (key, value) => Promise<any>) {
        app_id              = env.app_id;
        device_id           = env.device.platform;
        device_os           = `${env.os.name} ${env.os.version}`;
        device_model        = env.device.type;
        device_total_memory = ((<any>navigator).deviceMemory || 0);

        if (env.report) {
            report_user_device().then((r: string) => {
                const s = JSON.parse(r);
                init_flag = s.return_code === 1;
            });
        }
    }
    /**
     * user_register
     * @param data
     */
    export function report_user_register(data: user_register_data) {
        return _report('user_register', JSON.stringify(data));
    }
    /**
     * user_login
     * @param data
     */
    export function report_user_login(data: user_login_data) {
        return _report('user_login', JSON.stringify(data));
    }
    /**
     * user_offline
     * @param data
     */
    export function report_user_offline(data: user_offline_data) {
        return _report('user_offline', JSON.stringify(data));
    }
    /**
     * user_pay
     * @param data
     */
    export function report_user_pay(data: user_pay_data) {
        return _report('user_pay', JSON.stringify(data));
    }
    /**
     * user_device
     * @param data
     */
    export function report_user_device(data?: user_device_data) {
        if (data === undefined) {
            data = {
                app_id:                 app_id,
                device_id:              device_id,
                device_model:           device_model,
                device_os:              device_os,
                device_total_memory:    device_total_memory,
                register_time:          `${Date.now()}`
            }
        }

        return _report('user_device', JSON.stringify(data));
    }
    /**
     * device_use
     * @param data
     */
    export function report_device_use(data: device_use_data) {
        return _report('device_use', JSON.stringify(data));
    }
    /**
     * gi_data_change
     * @param data
     */
    export function report_gi_data_change(data: gi_data_change_data) {
        return _report('gi_data_change', JSON.stringify(data));
    }
    /**
     * 提交操作记录
     * * 需要先通过 record_page_trace 添加记录
     * * 在合适的时机才调用提交 - 目标操作出现错误 等情况
     */
    export function report_page_trace() {
        const dataStr: string = JSON.stringify(page_trace_list);

        // let dataStr = `[`;
        // for (let i = 0, len = page_trace_list.length -1; i<= len; i++) {
        //     const v = page_trace_list[i];
        //     dataStr += `{page:\"${v.page}\",stay_time:${v.stay_time}}`;
        //     if (i < len) {
        //         dataStr += `,`;
        //     }
        // }
        // dataStr += `]`;

        const data: page_trace_data = {
            time: `${Date.now()}`,
            device_id: device_id,
            app_id: app_id,
            data: dataStr
        }

        return _report('bi_page_trace', JSON.stringify(data));
    }
    /**
     * 添加操作记录
     * @page 操作名称
     * @stay_time 操作耗时
     */
    export function record_page_trace(page: string, stay_time: number) {
        const newStr = JSON.stringify({
            page,
            stay_time
        });

        const pageLen = page.length;
        const staytimeLen = (`${stay_time}`).length;

        // `{"page":\"${page}\","stay_time":${stay_time}},`
        // \"${page}\",${stay_time},...
        cur_page_trace_length += (2 + pageLen + 3 + staytimeLen + 1);

        if (_check_report_page_trace_length()) {
            report_page_trace();
            page_trace_list.length = 0;
        }

        page_trace_list.push(page);
        page_trace_list.push(stay_time);
    }
    function _check_report_page_trace_length() {
        // "data":"[...]" - 减去最后一个元素原本要添加的 ','
        // "time":"`${Date.now()}`",
        // "device_id":"device_id",
        // "app_id":"app_id",
        // { ... }
        const len = (
            1 
            + (8 + 1 + (cur_page_trace_length - 1) + 2) + 1
            + (8 + 13 + 1) + 1
            + (13 + device_id.length + 1) + 1
            + (10 + app_id.length + 1)
            + 1
        );

        return len > limit_report_length;
    }
    function _report(key: InterfaceType, data: string) {
        if (init_flag || key === 'user_device') {
            return PISYS.HTTP.post(`${baseUrl}${key}`, null, data, 'application/json', undefined, 1000);
        } else {
            return Promise.reject('');
        }
    }
}