import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class TalkingDataUpload implements IPlatformUpload {

    lastUser: IUploadUser;

    login(param: IUploadUser, _isCreateRole?: boolean): Json[] {
        let arr = [];

        if (!this.lastUser) {
            this.lastUser = this.createDefaultUser();
        }

        if (this.lastUser.userID !== param.userID) {
            arr.push({
                upload_type: "user",
                user: param.userID,
            });
        }

        if (this.lastUser.userType !== param.userType) {
            arr.push({
                upload_type: "type",
                type: param.userType,
            });
        }

        if (this.lastUser.userID !== param.userID
            || this.lastUser.userName !== param.userName
        ) {
            let userName = param.userID;
            if (param.userName) {
                userName += `:${param.userName}`;
            }
            arr.push({
                upload_type: "name",
                name: userName
            });
        }

        if (this.lastUser.roleLevel !== param.roleLevel) {
            arr.push({
                upload_type: "level",
                level: param.roleLevel
            });
        }

        if (this.lastUser.userSex !== param.userSex) {
            arr.push({
                upload_type: "sex",
                sex: param.userSex,
            });
        }

        if (this.lastUser.userAge !== param.userAge) {
            arr.push({
                upload_type: "age",
                age: param.userAge,
            });
        }

        if (this.lastUser.serverID !== param.serverID
            || this.lastUser.serverName !== param.serverName
        ) {
            let server = param.serverID;
            if (param.serverName) {
                server += `:${param.serverName}`;
            }
            arr.push({
                upload_type: "server",
                server
            });
        }

        this.lastUser = JSON.parse(JSON.stringify(param));
        return arr;
    }

    logout(): Json[] {
        return [];
    }

    pay(param: UploadPayParam): Json[] {

        let arr = [];

        let data: TalkingDataOnChargeRequest = {
            order_id: param.orderID,
            iap_id: param.goodsID,
            currency_amount: param.amount,
            currency_type: param.currencyType,
            virtual_currency_amount: param.goodsCount,
            payment_type: param.payment,
        };

        arr.push({
            upload_type: "on_charge_request",
            ...data
        });

        arr.push({
            upload_type: "on_charge_success",
            order_id: param.orderID,
        });

        return arr;
    }

    custom(type: string, value?: Json): Json {
        let r = null;

        if (!value) {
            return r;
        }

        switch (type) {
            case "on_reward":
                {
                    let p: IReward = {
                        upload_type: "on_reward",
                        currency_amount: value["currency_amount"],
                        reason: value["reason"],
                    };
                    r = p;
                }
                break;
            case "on_purchase":
                {
                    let p: IPurchase = {
                        upload_type: "on_purchase",
                        item: value["item"],
                        item_number: value["item_number"],
                        price_in_virtual_currency: value["price_in_virtual_currency"],
                    };
                    r = p;
                }
                break;
            case "on_use":
                {
                    let p: IUse = {
                        upload_type: "on_use",
                        item: value["item"],
                        item_number: value["item_number"],
                    };
                    r = p;
                }
                break;
            case "on_begin":
                {
                    let p: ITaskBegin = {
                        upload_type: "on_begin",
                        mission_id: value["mission_id"],
                    };
                    r = p;
                }
                break;
            case "on_completed":
                {
                    let p: ITaskCompleted = {
                        upload_type: "on_completed",
                        mission_id: value["mission_id"],
                    };
                    r = p;
                }
                break;
            case "on_failed":
                {
                    let p: ITaskFailed = {
                        upload_type: "on_failed",
                        mission_id: value["mission_id"],
                        cause: value["cause"],
                    };
                    r = p;
                }
                break;
            default:
                break;
        }
        return r;
    }

    private createDefaultUser(): IUploadUser {
        return {
            serverID: null,
            serverName: null,
            userID: null,
            userName: null,
            userType: null,
            userSex: null,
            userAge: -1,
            roleLevel: -1,
            roleVipLevel: -1,
            roleBalance: null,
            partyName: null,
            extend: null,
        }
    }
}

// TalkingData: 支付
interface TalkingDataOnChargeRequest {

    /**
     * 订单ID，最多64个字符。用于唯一标识一次交易。如果多次充值成功的 orderID 重复，只计算首次成功的数据，其他数据会认为重复数据丢弃。如果 Success 调用时传入的 orderID 在之前 Request 没有对应 orderID，则只记录充值次数，但不会有收入金额体现。
     */
    order_id: string;
    /**
     * 充值包 ID，最多 32 个字符。唯一标识一类充值包。例如：VIP3 礼包、500 元 10000 宝石包
     */
    iap_id: string;
    /**
     * 现金金额或现金等价物的额度 - Double
     */
    currency_amount: number;
    /**
     * 请使用国际标准组织 ISO4217 中规范的 3 位字母代码标记货币类型。例：人民币 CNY；美元 USD；欧元 EUR；如果您使用其他自定义等价物作为现金，亦可使用 ISO4217 中没有的3位字母组合传入货币型，我们会在报表页面中提供汇率设定功能
     */
    currency_type: string;
    /**
     * 虚拟币金额 - Double
     */
    virtual_currency_amount: number;
    /**
     * 支付的途径，最多16个字符。例如：“支付宝”、“苹果官方”、“XX 支付SDK
     */
    payment_type: string;
}

/**
 * 赠予虚拟币
 */
interface IReward {
    upload_type: "on_reward",
    currency_amount: number, // 现金金额或现金等价物的额度 - Double
    reason: string, // 赠送虚拟币原因/类型。格式：32 个字符内的中文、空格、英文、数字。不要带有任何开发中的转义字符，如斜杠。注意：最多支持 100 种不同原因。
}

/**
 * 记录付费点
 */
interface IPurchase {
    upload_type: "on_purchase",
    item: string,           // 某个消费点的编号，最多 32 个字符。
    item_number: number,    // 消费数量 - Int
    price_in_virtual_currency: number,  // 虚拟币单价 - Double
};

/**
 * 消耗物品或服务等
 */
interface IUse {
    upload_type: "on_use",
    item: string,           // 某个消费点的编号，最多 32 个字符。
    item_number: number,    // 消费数量 - Int
};

/**
 * 进入任务
 */
interface ITaskBegin {
    upload_type: "on_begin",
    mission_id: string, // 任务、关卡或副本的编号，最多支持 32 个字符。此处可填写 ID，别名可在报表中编辑
}

/**
 * 任务完成
 */
interface ITaskCompleted {
    upload_type: "on_completed",
    mission_id: string, // 任务、关卡或副本的编号，最多支持 32 个字符。此处可填写 ID，别名可在报表中编辑
}

/**
 * 任务失败
 */
interface ITaskFailed {
    upload_type: "on_failed",
    mission_id: string, // 任务、关卡或副本的编号，最多支持 32 个字符。此处可填写 ID，别名可在报表中编辑。
    cause: string,      // 失败原因，非必选。最多支持 16个字符。共支持100种原因。
}