import { Json } from "pi_tools/type";
import { UploadPayParam } from "./pay";

/**
 * 数据上报 上下文
 */
export interface IUploadContext {

    /**
     * 返回打包时注册的 上报相关 的 SDK名 数组
     * 
     * @Note 默认注册多少个SDK，就上报多少个SDK；
     */
    getSdkArray(): string[];

    /**
     * 取最新设置的用户信息
     */
    getLastUserInfo(): IUploadUser;

    /**
     * 更新用户信息
     */
    updateUserInfo(info: IUploadUser);

    /**
     * 上报 注册 信息
     */
    register();

    /**
     * 上报 登录 信息
     */
    login();
    /**
     * 上报 激活 信息
     */
    reportActivation()
    /**
     * 登出 上报
     * 
     * 当 登出成功时 调用
     * 
     */
    logout();
    /**
     * 支付成功 上报
     * 
     * 当 支付成功时 调用
     * 
     */
    pay(info: UploadPayParam);

    /**
     * 上报 自定义 数据
     * @note 具体的type和value的数据类型，请阅读 相关的平台对应的ts文件
     * 
     * @example 比如 tabdb的自定义事件：
     *     custom("event", {
     *         event_code: "1",
               properties: "abc",
     *     });
     */
    custom(type: string, value?: Json);
}

/**
 * 用户 信息
 */
export interface IUploadUser {

    /**
     * 区服 信息
     * 
     * @note 有些平台，没有同时 支持 ID 和 name，实现会进行 合并
     *   server = `${serverID}:${serverName}`，冒号视name或id是否为空决定；
     */
    serverID?: string,       // 区服 ID，纯英文，唯一
    serverName?: string,    // 区服名；默认值：""
    /**
     * 玩家 (角色) 信息
     * 
     * @note 有些平台，没有同时 支持 ID 和 name，实现会进行 合并
     *   server = `${userID}:${userName}`，冒号视name或id是否为空决定；
     */
    userID?: string,         // 玩家（角色）ID，纯英文，唯一
    userName?: string,      // 玩家（角色）名；默认值：""

    userType?: string,     // 账号类型（来自哪个平台），比如：qq，wx，weibo 等；默认值：""

    userSex?: "male" | "female" | "unknown";    // 玩家 性别；默认值："unknown"
    userAge?: number,                           // 玩家 年龄；默认值：0

    roleLevel?: number,      // 角色等级；默认值：0
    roleVipLevel?: number,   // 用户 vip 充值 等级；默认值：0
    roleBalance?: string,    // 角色 余额；默认值：""

    partyId?: number,        // 帮派id；默认值：0
    partyName?: string,      // 帮派名；默认值：""
    createTime?: number,     // 创建时间；时间戳 默认值：0
    levelUpTime?: number,    // 升级时间；时间戳 默认值：0

    extend?: string,         // 扩展串；默认值：""

    /**
     * 设备 信息
     * 
     * @note BI 新增字段 可从 PISYS.Native 中获取
     */
    device?: string,         // 设备ID  PISYS.Native.SystemInfo.getDeviceID()
    channel?: string,        // 渠道  PISYS.Native.App.getFlavorName()
    version?: string,        // APP 版本号  PISYS.Native.App.getVersionCode() 
    network?: string,        // 网络环境 (2G/3G/4G/5G/WIFI)  PISYS.Native.Network.getType()

    ip?: string,             // 本机 IP
    uuid?: string,           // 用户ID 唯一标识（不分区服）

    /**
     * quick（咪咕） 新增
     */
    data_level?: number,       // 玩家等级 默认值：0
    data_total_money?: number, // 商铺累计收益 默认值：0
    data_blueprint?: number,   // 装备图鉴数量 默认值：0
    // key?: string,              // 签名key
    channel_role_sign?: string, // 签名

    /**
     * 弎湘 quickh5
     */
    fightingCapacity?: string,   // 战力
    reincarnation?: string,  // 转生 ?不知道是啥
    profession?: string,  // 角色在游戏内的职业

    /**
     * h5 渠道必传
     */
    uploadType?: UploadType // 上报渠道


    /**
     * 大千
     */
    friendList?: {
        nexusId: number,   // 关系 id，（0：无，1：夫妻，2：结拜，3：情侣,4：师徒，5：仇人，自定义从 6 推后）
        roleid: number,    // 好友角色关系 id
        nexusName: string, // 关系名，对应关系 id（如：夫妻）
        intimacy: number   // 亲密度，无传 0
    }[],       //朋友关系
    banlance?: {
        balanceid: number,	 //货币代号 id
        balancename: string, //货币名称
        balancenum: number	 //货币余额
    }[],        //钱包
    professionId?: number,   //当前登录玩家的职业 ID
    partyroleid?: number,    //帮派称号 ID
    partyrolename?: string   //帮派称号名称
    task_id?: number,        //	自定义任务id 
    task_name?: string,      //	任务名
    task_status?: number,    //	完成任务状态 1 成功 2 失败 
    honor_id?: number,       //	自定义荣誉id 
    honor_name?: string      //	荣誉名称
    payTotal?: number        // 累计充值金额 单位 (元)
    power?: string           //战力
}

/**
 * 上报 类型
 * Android/iOS 底层的名字
 */
export enum UploadType {
    yuchen = 'yc',                // 煜辰
    mengyou = 'my',               // 萌友
    dongwan = 'dw',               // 动玩
    quicksdk = 'quicksdk',        // quicksdk
    ohy_sdk = 'ohy_sdk',          // ohy_sdk
    talkingdata = 'talkingdata',  // talkingdata
    taptap = 'tapdb',             // taptap
    jrtt_log = 'jrtt_log',        // 今日头条
    muxue = 'muxue',              // 暮雪
    quicksdkio = 'quicksdkio',    // quick海外
    quicksdkios = 'qkios',        // quick ios
    daqian = 'daqian',            // 大千 sdk
    baidu = 'baidu',            // 百度 sdk
    txgdt = 'txgdt',            // 腾讯广点通 sdk
    ios_cdqxz = 'ios_cdqxz',        //辰东ios
    bi = 'bi',   // BI

    // h5渠道
    ronghe = 'ronghe',                  // 大千融合
    quick_h5 = 'quick_h5',              // quick h5
    sanxiang = 'sanxiang',              // 弎湘
    qq_game = 'qq_game',                // QQ大厅
    qingcheng = 'qingcheng',            // 倾程
    qingcheng_vivo = 'qingcheng_vivo',  // 倾程 vivo
    qingcheng_zfb = 'qingcheng_zfb',    // 倾程 支付宝
    qingcheng_mt = 'qingcheng_mt',      // 倾程 美团
    ks_ad = 'ks_ad',      // 快手特别ad渠道
}